{% load i18n %}
{% load bootstrap %}
{% load static %}

<!-- structure list -->
<div role="tabpanel" class="tab-pane" id="jump-gates">
    <div class="table-responsive">
        <table id="tab_jump_gates" style="width: 100%" class="table table-striped table-compact">
            <thead>
                <tr>
                    <th></th>
                    <th>{% translate "Owner" %}</th>
                    <th>{% translate "Location" %}</th>
                    <th>{% translate "Name" %}</th>
                    <th>{% translate "Last Online" %}</th>
                    <th>{% translate "Fuel Expires" %}</th>
                    <th>{% translate "Liquid Ozone (units)" %}</th>
                </tr>
            </thead>
        </table>
        <p class="text-muted">
            {% translate "Reinforced structures (except POCOs) are highlighted in red" %}
        </p>
    </div>
</div>

<!-- Modals -->
<div class="modal" id="modalUpwellDetails" tabindex="-1" role="dialog" aria-labelledby="{% translate 'modalUpwellDetails '%}">
    <div class="modal-dialog" role="document">
        <div class="modal-content" id="modalUpwellDetailsContent">
            <div class="modal-header">
                <button type="button" class="close" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">&times;</span></button>
                <h4 class="modal-title">Loading...</h4>
            </div>
            <div class="modal-body">
            </div>
        </div>
    </div>
</div>
