# Generated by Django 3.1.6 on 2021-04-30 14:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("structures", "0023_structure_fittings"),
    ]

    operations = [
        migrations.AlterField(
            model_name="evecategory",
            name="name",
            field=models.CharField(help_text="Eve Online name", max_length=255),
        ),
        migrations.AlterField(
            model_name="evecategory",
            name="name_de",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for German",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="evecategory",
            name="name_ko",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Korean",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="evecategory",
            name="name_ru",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Russian",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="evecategory",
            name="name_zh",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Chinese",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="eveconstellation",
            name="name",
            field=models.CharField(help_text="Eve Online name", max_length=255),
        ),
        migrations.AlterField(
            model_name="eveconstellation",
            name="name_de",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for German",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="eveconstellation",
            name="name_ko",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Korean",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="eveconstellation",
            name="name_ru",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Russian",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="eveconstellation",
            name="name_zh",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Chinese",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="evegroup",
            name="name",
            field=models.CharField(help_text="Eve Online name", max_length=255),
        ),
        migrations.AlterField(
            model_name="evegroup",
            name="name_de",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for German",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="evegroup",
            name="name_ko",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Korean",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="evegroup",
            name="name_ru",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Russian",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="evegroup",
            name="name_zh",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Chinese",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="evemoon",
            name="name",
            field=models.CharField(help_text="Eve Online name", max_length=255),
        ),
        migrations.AlterField(
            model_name="evemoon",
            name="name_de",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for German",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="evemoon",
            name="name_ko",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Korean",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="evemoon",
            name="name_ru",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Russian",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="evemoon",
            name="name_zh",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Chinese",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="eveplanet",
            name="name",
            field=models.CharField(help_text="Eve Online name", max_length=255),
        ),
        migrations.AlterField(
            model_name="eveplanet",
            name="name_de",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for German",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="eveplanet",
            name="name_ko",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Korean",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="eveplanet",
            name="name_ru",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Russian",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="eveplanet",
            name="name_zh",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Chinese",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="everegion",
            name="name",
            field=models.CharField(help_text="Eve Online name", max_length=255),
        ),
        migrations.AlterField(
            model_name="everegion",
            name="name_de",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for German",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="everegion",
            name="name_ko",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Korean",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="everegion",
            name="name_ru",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Russian",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="everegion",
            name="name_zh",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Chinese",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="evesolarsystem",
            name="name",
            field=models.CharField(help_text="Eve Online name", max_length=255),
        ),
        migrations.AlterField(
            model_name="evesolarsystem",
            name="name_de",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for German",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="evesolarsystem",
            name="name_ko",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Korean",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="evesolarsystem",
            name="name_ru",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Russian",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="evesolarsystem",
            name="name_zh",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Chinese",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="evetype",
            name="name",
            field=models.CharField(help_text="Eve Online name", max_length=255),
        ),
        migrations.AlterField(
            model_name="evetype",
            name="name_de",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for German",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="evetype",
            name="name_ko",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Korean",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="evetype",
            name="name_ru",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Russian",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="evetype",
            name="name_zh",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Chinese",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="structureservice",
            name="name_de",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for German",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="structureservice",
            name="name_ko",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Korean",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="structureservice",
            name="name_ru",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Russian",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="structureservice",
            name="name_zh",
            field=models.CharField(
                blank=True,
                help_text="Eve Online name localized for Chinese",
                max_length=255,
            ),
        ),
    ]
