# Generated by Django 2.2.13 on 2020-08-24 13:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("auth", "0011_update_proxy_permissions"),
        ("structures", "0012_disable_pings"),
    ]

    operations = [
        migrations.RenameField(
            model_name="owner",
            old_name="has_pings_enabled",
            new_name="has_default_pings_enabled",
        ),
        migrations.RenameField(
            model_name="webhook",
            old_name="has_pings_enabled",
            new_name="has_default_pings_enabled",
        ),
        migrations.RemoveField(
            model_name="structure",
            name="next_reinforce_weekday",
        ),
        migrations.RemoveField(
            model_name="structure",
            name="reinforce_weekday",
        ),
        migrations.AddField(
            model_name="owner",
            name="ping_groups",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="Groups to be pinged for each notification - ",
                to="auth.Group",
            ),
        ),
        migrations.AddField(
            model_name="webhook",
            name="ping_groups",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="Groups to be pinged for each notification - ",
                to="auth.Group",
            ),
        ),
        migrations.AlterField(
            model_name="structure",
            name="next_reinforce_apply",
            field=models.DateTimeField(
                blank=True,
                default=None,
                help_text="The requested change to reinforce_hour that will take effect at the time shown by next_reinforce_apply",
                null=True,
            ),
        ),
    ]
